<?php
/**
 * MailerLite API Helper
 */
if ( ! defined( 'ABSPATH' ) ) {
	die();
}

if ( ! class_exists( 'Auto_Mail_MailerLite_API' ) ) :

	class Auto_Mail_MailerLite_API {

		/**
		 * Api Key
		 *
		 * @var String
		 */
		protected $api_key;

		/**
		 * End point
		 *
		 * @var String
		 */
		private $_end_point = 'https://api.mailerlite.com/api/v2/';


		public function __construct( $_api_key ) {
			$this->api_key = $_api_key;
		}

		/**
		 * Perform API remote request
		 *
		 * @param String       $path - relative api path to the $_end_point
		 * @param String       $method - allowed methods are GET, POST, PUT, DELETE
		 * @param String|Array $input - the post data
		 *
		 * @return WP_Error|Array
		 */
		private function _do_request( $path, $method, $input ) {

			$called_url = $this->_end_point . $path;
			$ssl_verify = true;
			if ( strtoupper( substr( PHP_OS, 0, 3 ) ) === 'WIN' ) {
				// Windows only over-ride
				$ssl_verify = false;
			}

			$args = array(
				'method'    => $method,
				'sslverify' => $ssl_verify,
				'headers'   => array(
					'X-MailerLite-ApiKey' => $this->api_key,
					'Content-Type'        => 'application/json',
				),
			);

			$args['body'] = $input;

			$response = wp_remote_request( $called_url, $args );

			$data = wp_remote_retrieve_body( $response );

			if ( is_wp_error( $data ) ) {
				return $data;
			}

			return json_decode( $data, true );
		}

		/**
		 * GET Request
		 */
		private function _get( $path, $input = array() ) {
			return $this->_do_request( $path, 'GET', $input );
		}

		/**
		 * PUT Http request
		 */
		private function _put( $path, $input = array() ) {
			return $this->_do_request( $path, 'PUT', wp_json_encode( $input ) );
		}

		/**
		 * POST Http request
		 */
		private function _post( $path, $input = array() ) {
			return $this->_do_request( $path, 'POST', wp_json_encode( $input ) );
		}

		/**
		 * DELETE Http request
		 */
		private function _delete( $path, $input = array() ) {
			return $this->_do_request( $path, 'DELETE', $input );
		}

		/**
		 * List Groups
		 *
		 * @return Array|WP_Error
		 */
		public function list_groups( $offset = 0 ) {
			return $this->_get( 'groups?offset=' . intval( $offset ) );
		}

		/**
		 * Add Subscriber
		 *
		 * @param Integer $group_id - the group id
		 * @param Array   $subscriber_data - An array containing the keys email and fields(name,value)
		 */
		public function add_subscriber( $group_id, $subscriber_data, $resubscribe = 0 ) {
			$subscriber_data['resubscribe'] = $resubscribe;
			$path                           = 'groups/' . $group_id . '/subscribers';

			$res = $this->_post( $path, $subscriber_data );

			return $res;
		}

		/**
		 * Update Subscriber
		 *
		 * @param Integer $sub_id - the subscriber id
		 * @param Array   $subscriber_data - An array containing the keys email and fields(name,value)
		 */
		public function update_subscriber( $email, $subscriber_data ) {
			$path = 'subscribers/' . $email;

			$res = $this->_put( $path, $subscriber_data );

			return $res;
		}

		/**
		 * Get Subscriber groups
		 *
		 * @param String $email
		 */
		public function get_subscriber( $email ) {
			$path = 'subscribers/' . $email . '/groups';

			$res = $this->_get( $path );

			return $res;
		}

		/**
		 * Add custom field
		 *
		 * @param Array $field_data (title, type)
		 */
		public function add_custom_field( $field_data ) {
			$path = 'fields';

			$res = $this->_post( $path, $field_data );

			return $res;
		}

		/**
		 * Get custom field
		 */
		public function get_custom_field() {
			$path = 'fields';
			$res  = $this->_get( $path );

			return $res;
		}

        /**
         * subscribe user email into list
         *
         * @return array
         */
        public function subscribe_to_list($email, $list_id){
            $url = 'https://api.mailerlite.com/api/v1/subscribers/'.$list_id.'/';

            // mailerlite api need http bearer authentication
            $header = array(
                'X-MailerLite-ApiKey' => $this->api_key,
                'content-type'  => 'application/json'
            );

            $params = array(
                "email" => $email
            );

            $body = json_encode($params);

            // make put request
            $return_data  = $this->make_post_request( $url, $header, $body );

            $result = json_decode($return_data, true);

            return $result;
        }

        /**
         * Make HTTP POST Resuest
         *
         * @param  string $url
         * @return string
         */
        public function make_post_request( $url, $headers = array(), $body) {
            // build http post request args
            $args = array(
                'headers' => $headers,
                'body'    => $body,
                'method'  => 'POST',
                'timeout' => 45,
            );

            $request = wp_remote_post( $url, $args );

            // retrieve the body from the raw response
            $result = wp_remote_retrieve_body( $request );

            // log error messages
            if ( is_wp_error( $request ) ) {
                return false;
            }

            if ( $request['response']['code'] != 200 ) {
                return false;
            }

            return $result;

        }
	}
endif;
