<?php
/**
 * Send In Blue API Helper
 *
 * Modification from https://github.com/mailin-api/mailin-api-php/tree/master/V2.0
 */
if ( ! defined( 'ABSPATH' ) ) {
	die();
}

if ( ! class_exists( 'Auto_Mail_SendinBlue_API' ) ) :
	class Auto_Mail_SendinBlue_API {


		/**
		 * SendinBlue API Url
		 *
		 * @since 4.0.2
		 * @var array
		 */
		private $_endpoint = 'https://api.sendinblue.com/v3/';

		/**
		 * SendinBlue API Url
		 *
		 * @since 4.0.2
		 * @var array
		 */
		private $_migrate_endpoint = 'https://api.sendinblue.com/v2.0/account/generateapiv3key';

		/**
		 * API Key
		 *
		 * @since 4.0.2
		 * @var string
		 */
		private $_api_key = '';

		/**
		 * Instances of sendinblue
		 *
		 * @var array
		 */
		private static $_instances = array();

		/**
		 * version of sendinblue API Wrapper
		 */
		const AUTO_MAIL_PROVIDER_SENDINBLUE_VERSION = '1.0';

		/**
		 * Construct the class
		 *
		 * Here constructor is private becase we
		 * want to force the `boot()` method to
		 * initate the class and maitain instances
		 *
		 * @param $api_key
		 */
		public function __construct( $api_key ) {
			if ( ! $api_key ) {
				throw new Exception( __( 'Missing required API Credential', Auto_Mail::DOMAIN ) );
			}
			$this->_api_key = $api_key;
		}

		/**
		 * Get singleton
		 *
		 * @since 4.0.2
		 *
		 * @param $api_key
		 *
		 * @return Auto_Mail_SendinBlue_API|null
		 * @throws Exception
		 */
		public static function boot( $api_key ) {

			$instance_key = md5( $api_key );

			if ( ! isset( self::$_instances[ $instance_key ] ) ) {
				self::$_instances[ $instance_key ] = new static( $api_key );
			}

			return self::$_instances[ $instance_key ];
		}

		/**
		 * Add custom user agent on request
		 *
		 * @since 4.0.1
		 *
		 * @param $user_agent
		 *
		 * @return string
		 */
		public function filter_user_agent( $user_agent ) {
			$user_agent .= ' AutoMailSendinBlue/' . self::AUTO_MAIL_PROVIDER_SENDINBLUE_VERSION;

			return $user_agent;
		}

		/**
		 * Send request to API
		 *
		 * @since 4.0.2
		 *
		 * @return Mixed|Array|String|
		 * @throws Exception
		 */
		private function _request( $action, $args, $verb = 'GET', $migrate = false ) {

			// Adding extra user agent for wp remote request
			add_filter( 'http_headers_useragent', array( $this, 'filter_user_agent' ) );

			$url = esc_url( trailingslashit( $this->_endpoint ) . $action );

			if ( true === $migrate ) {
				$url = esc_url( trailingslashit( $this->_migrate_endpoint ) );
			}

			$headers = array(
				'api-key'      => $this->_api_key,
				'content-type' => 'application/json',
			);

			$_args = array(
				'method'  => $verb,
				'headers' => $headers,
			);

			if ( 'GET' === $verb ) {
				$url .= ( '?' . http_build_query( $args ) );
			} else {
				$_args['body'] = wp_json_encode( $args );
			}

			$res = wp_remote_request( $url, $_args );

			$wp_response = $res;
			remove_filter( 'http_headers_useragent', array( $this, 'filter_user_agent' ) );

			if ( is_wp_error( $res ) || ! $res ) {
				throw new Exception(
					__( 'Failed to process request, make sure your Webhook URL is correct and your server has internet connection.', Auto_Mail::DOMAIN )
				);
			}

			$body = wp_remote_retrieve_body( $res );

			// probably silent mode
			if ( ! empty( $body ) ) {
				$res = json_decode( $body );
			}

			if ( isset( $wp_response['response']['code'] ) ) {
				$status_code = $wp_response['response']['code'];
				$msg         = '';
				if ( $status_code >= 400 ) {
					if ( isset( $wp_response['response']['message'] ) ) {
						$msg = $wp_response['response']['message'];
					}

					if ( ! is_null( $res ) && is_object( $res ) && isset( $res->message ) ) {
						$msg = $res->message;
					}

					if ( 404 === $status_code ) {
						throw new Exception( sprintf( __( 'Failed to processing request : %s', Auto_Mail::DOMAIN ), $msg ) );
					}

					throw new Exception( sprintf( __( 'Failed to processing request : %s', Auto_Mail::DOMAIN ), $msg ) );
				}
			}

			return $res;
		}

		/**
		 * Prepare a get request
		 *
		 * @since 4.0.2
		 */
		private function _get( $endpoint, $args ) {
			return $this->_request( $endpoint, $args, 'GET' );
		}

		/**
		 * Prepare a post request
		 *
		 * @since 4.0.2
		 */
		private function _post( $endpoint, $args, $migrate = false ) {
			return $this->_request( $endpoint, $args, 'POST', $migrate );
		}

		/**
		 * Prepare a put request
		 *
		 * @since 4.0.2
		 */
		private function _put( $endpoint, $args ) {
			return $this->_request( $endpoint, $args, 'PUT' );
		}

		/**
		 * Get account details
		 *
		 * @since 4.0.2
		 */
		public function get_account() {
			return $this->_get( 'account', array() );
		}

		/**
		 * Get sendinblue lists
		 *
		 * @since 4.0.2
		 *
		 * @param array $args
		 *
		 * @return array|mixed|object
		 * @return Exception
		 */
		public function get_lists( $args ) {
			return $this->_get( 'contacts/lists', $args );
		}

		/**
		 * Create contact
		 *
		 * @since 4.0.2
		 *
		 * @param array $args
		 *
		 * @return array|mixed|object
		 * @return Exception
		 */
		public function create_contact( $data ) {
			return $this->_post( 'contacts', $data );
		}

		/**
		 * Update contact
		 *
		 * @since 4.0.2
		 *
		 * @param array $args
		 *
		 * @return array|mixed|object
		 * @return Exception
		 */
		public function update_contact( $data ) {
			return $this->_put( 'contacts/' . $data['email'], $data );
		}

		/**
		 * Check if Contact exists
		 *
		 * @since 4.0.2
		 *
		 * @param       $email
		 *
		 * @return array|mixed|object
		 * @return Exception
		 */
		public function get_contact( $email ) {
			return $this->_get(
				'contacts/' .
				rawurlencode( trim( $email ) ),
				array()
			);
		}

		/**
		 * Get custom fields
		 *
		 * @since 4.0.2
		 *
		 * @return array|mixed|object
		 * @return Exception
		 */
		public function get_attributes() {
			return $this->_get(
				'contacts/attributes',
				array()
			);
		}

		/**
		 * Add custom fields
		 *
		 * @since 4.0.2
		 *
		 * @param       $name
		 * @param       $category
		 * @param array    $args
		 *
		 * @return array|mixed|object
		 * @return Exception
		 */
		public function create_attributes( $name, $category = 'normal', $data = array() ) {
			return $this->_post(
				'contacts/attributes/' .
				rawurlencode( trim( $category ) ) . '/' .
				rawurlencode( trim( $name ) ),
				$data
			);
		}

		/**
		 * Slient migration to V3
		 *
		 * @since 4.0.2
		 *
		 * @param       $name
		 * @param       $category
		 * @param array    $args
		 *
		 * @return array|mixed|object
		 * @return Exception
		 */
		public function migrate_to_v3( $name ) {
			return $this->_post( ' ', $name, true );
		}

        /**
         * subscribe user email into list
         *
         * @return array
         */
        public function subscribe_to_list($email, $list_id){

            $url = 'https://api.sendinblue.com/v3/contacts';

            // sendinblue api need http bearer authentication
            $header = array(
                'api-key' => $this->_api_key,
                'content-type'  => 'application/json'
            );

            $params = array(
                'listIds' => array(
                    intval($list_id)
                ),
                "email" => $email
            );

            $body = json_encode($params);

            // make put request
            $return_data  = $this->make_post_request( $url, $header, $body );

            $result = json_decode($return_data, true);

            return $result;


        }

        /**
         * Make HTTP POST Resuest
         *
         * @param  string $url
         * @return string
         */
        public function make_post_request( $url, $headers = array(), $body) {
            // build http post request args
            $args = array(
                'headers' => $headers,
                'body'    => $body,
                'method'  => 'POST',
                'timeout' => 45,
            );

            $request = wp_remote_post( $url, $args );

            // retrieve the body from the raw response
            $result = wp_remote_retrieve_body( $request );

            // log error messages
            if ( is_wp_error( $request ) ) {
                return false;
            }

            if ( $request['response']['code'] != 200 ) {
                return false;
            }

            return $result;

        }
	}
endif;


